@@ -13,19 +13,34 @@ import (
 	"unsafe"
 )
 
-func (t *Tracer) exitHandler(pid int, regs *syscall.PtraceRegs) (err error) {
-	inst := Inst(regs)
+func (t *Tracer) getArgsFromStorehouse(pid, inst int) ([]uint64, error) {
 	v, ok := t.storehouse.Get(pid, inst)
 	if !ok {
-		return fmt.Errorf("cannot get the arguments from storehouse because of neverseen: pid: %v, inst: %v", pid, inst)
+		return nil, fmt.Errorf("cannot get the arguments from storehouse because of neverseen: pid: %v, inst: %v", pid, inst)
 	}
 	t.storehouse.Remove(pid, inst)
-	args := v.([]uint64)
+	return v.([]uint64), nil
+}
+
+func (t *Tracer) saveArgsToStorehouse(pid, inst int, args []uint64) {
+	t.storehouse.Save(pid, inst, args)
+}
+
+func (t *Tracer) exitHandler(pid int, regs *syscall.PtraceRegs) (err error) {
+	//t.log.Infof("exitHandler: pid: %v,inst: %v", pid, inst(regs))
+	//defer t.log.Infof("exitHandler: pid: %v,inst: %v: end", pid, inst(regs))
+	inst := inst(regs)
 	switch inst {
 	case syscall.SYS_SOCKET:
-		fd, errno := ReturnValueInt(regs)
+		//t.log.Tracef("exitHandler: SOCKET: %v, inst: %v", pid, inst)
+		args, err := t.getArgsFromStorehouse(pid, inst)
+		if err != nil {
+			t.log.Infoln(err)
+			return nil
+		}
+		fd, errno := returnValueInt(regs)
 		if errno != 0 {
-			logrus.Tracef("socket error: pid: %v, errno: %v", pid, errno)
+			logrus.Infof("socket error: pid: %v, errno: %v", pid, errno)
 			return nil
 		}
 		socketInfo := SocketMetadata{
@@ -37,7 +52,13 @@ func (t *Tracer) exitHandler(pid int, regs *syscall.PtraceRegs) (err error) {
 		t.saveSocketInfo(pid, fd, socketInfo)
 		t.log.Tracef("new socket (%v): pid: %v, fd %v", t.network(&socketInfo), pid, fd)
 	case syscall.SYS_FCNTL:
+		//t.log.Tracef("exitHandler: FCNTL: %v, inst: %v", pid, inst)
 		// syscall.SYS_FCNTL can be used to duplicate the file descriptor.
+		args, err := t.getArgsFromStorehouse(pid, inst)
+		if err != nil {
+			t.log.Traceln(err)
+			return nil
+		}
 		switch args[1] {
 		case syscall.F_DUPFD, syscall.F_DUPFD_CLOEXEC:
 		default:
@@ -49,7 +70,7 @@ func (t *Tracer) exitHandler(pid int, regs *syscall.PtraceRegs) (err error) {
 			logrus.Tracef("syscall.F_DUPFD: socketInfo cannot found: pid: %v, fd: %v", pid, fd)
 			return nil
 		}
-		newFD, errno := ReturnValueInt(regs)
+		newFD, errno := returnValueInt(regs)
 		if errno != 0 {
 			logrus.Tracef("socket error: pid: %v, errno: %v", pid, errno)
 			return nil
@@ -57,6 +78,7 @@ func (t *Tracer) exitHandler(pid int, regs *syscall.PtraceRegs) (err error) {
 		t.saveSocketInfo(pid, newFD, *socketInfo)
 		t.log.Tracef("SYS_FCNTL: copy %v -> %v", fd, newFD)
 	case syscall.SYS_CLOSE:
+		//t.log.Tracef("exitHandler: CLOSE: %v, inst: %v", pid, inst)
 		// we do not need to know if it succeeded
 		fd := Argument(regs, 0)
 		t.removeSocketInfo(pid, int(fd))
@@ -65,10 +87,20 @@ func (t *Tracer) exitHandler(pid int, regs *syscall.PtraceRegs) (err error) {
 }
 
 func (t *Tracer) entryHandler(pid int, regs *syscall.PtraceRegs) (err error) {
-	inst := Inst(regs)
-	args := Arguments(regs)
-	t.storehouse.Save(pid, inst, args)
-	switch inst {
+	//t.log.Infof("entryHandler: pid: %v,inst: %v", pid, inst(regs))
+	//defer t.log.Infof("entryHandler: pid: %v,inst: %v: end", pid, inst(regs))
+	args := arguments(regs)
+	switch inst(regs) {
+	//case syscall.SYS_CLONE:
+	//	//t.log.Tracef("entryHandler: clone: %v", pid)
+	//	newRegs := *regs
+	//	setArgument(&newRegs, 0, args[0] & ^uint64(syscall.CLONE_UNTRACED))
+	//	if err = ptraceSetRegs(pid, &newRegs); err != nil {
+	//		return err
+	//	}
+	case syscall.SYS_SOCKET, syscall.SYS_FCNTL:
+		//t.log.Tracef("entryHandler: SOCKET, FCNTL: %v, inst: %v", pid, inst(regs))
+		t.saveArgsToStorehouse(pid, inst(regs), args)
 	case syscall.SYS_CONNECT, syscall.SYS_SENDTO:
 		fd := args[0]
 		t.log.Tracef("syscall.SYS_CONNECT: pid: %v, fd: %v", pid, fd)
@@ -83,7 +115,7 @@ func (t *Tracer) entryHandler(pid int, regs *syscall.PtraceRegs) (err error) {
 			pSockAddr, sockAddrLen uint64
 			orderSockAddrLen       int
 		)
-		switch Inst(regs) {
+		switch inst(regs) {
 		case syscall.SYS_CONNECT:
 			t.log.Tracef("syscall.SYS_CONNECT")
 			pSockAddr = args[1]
@@ -206,8 +238,8 @@ func pokeAddrToArgument(pid int, regs *syscall.PtraceRegs, bAddrToPoke []byte, p
 		return nil
 	}
 	newRegs := *regs
-	SetArgument(&newRegs, orderSockAddrLen, uint64(len(bAddrToPoke)))
-	if err = syscall.PtraceSetRegs(pid, &newRegs); err != nil {
+	setArgument(&newRegs, orderSockAddrLen, uint64(len(bAddrToPoke)))
+	if err = ptraceSetRegs(pid, &newRegs); err != nil {
 		return fmt.Errorf("set addr len for connect: %w", err)
 	}
 	return nil
